!---------------------------------------------------------------------------------------
! FLEXINVERT: interpz
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> interpz 
!!
!! Purpose:    Interpolates concentration data vertically to the flexpart levels
!!
!! Interface:
!!
!!    Inputs
!!             nx      -  number of grid cells longitude
!!             ny      -  number of grid cells latitude
!!             nz      -  number of vertical layers in input
!!             zkind   -  indicator of height reference (0 = sea level, 1 = ground level) 
!!             geoh    -  geopotential height mid-layer in input (metres above sea level)
!!             gridin  -  input data
!!             gridout -  output data (zero array on input)
!!
!!    Note:    outheight is for top of layers in metres above surface so need to adjust
!!             to metres above sea level using orography
!!
!---------------------------------------------------------------------------------------

subroutine interpz(nx, ny, nz, zkind, midpoints, lon, lat, geoh, gridin, gridout)

  use mod_var

  implicit none

  integer,                       intent(in)     :: nx, ny, nz, zkind
  logical,                       intent(in)     :: midpoints
  real, dimension(nx),           intent(in)     :: lon
  real, dimension(ny),           intent(in)     :: lat
  real, dimension(nx,ny,nz),     intent(in)     :: geoh
  real, dimension(nx,ny,nz),     intent(in)     :: gridin
  real, dimension(nx,ny,nzgrid), intent(in out) :: gridout

  integer                                       :: ix, jy, kz
  integer                                       :: nn, ii, jj
  real                                          :: hadj, dz1, dz2, ddx, ddy
  real, dimension(nzgrid)                       :: halfh
  !*** TESTING
!  real, dimension(nx,ny,nzgrid)                 :: htest

  do nn = 1, nzgrid
    if ( nn.eq.1 ) then
      halfh(nn) = 0.5*outheight(nn)
    else 
      halfh(nn) = 0.5*(outheight(nn-1) + outheight(nn))
    endif
  end do
!  print*, 'outheight = ',outheight
!  print*, 'halfh = ',halfh

  ! flexpart coordinates are western and southern boundary
  if ( midpoints ) then
    ddy = dy/2.
    ddx = dx/2.
  else 
    ddy = 0.
    ddx = 0.
  endif
!  print*, 'ddx, ddy = ',ddx,ddy
     
  do jy = 1, ny
    jj = minloc(gbl_lat,dim=1,mask=abs(gbl_lat+ddy-lat(jy)).eq.minval(abs(gbl_lat+ddy-lat(jy))))
    do ix = 1, nx     
      ii = minloc(gbl_lon,dim=1,mask=abs(gbl_lon+ddx-lon(ix)).eq.minval(abs(gbl_lon+ddx-lon(ix))))
      if ( zkind.eq.0 ) then
        ! input is relative to sea level
        hadj = max(orog(ii,jj),0.)
      else
        ! input is relative to ground level
        hadj = 0.
      endif
!      if (jy.eq.ny/2.and.ix.eq.nx/2) print*, 'gbl_lat, lat, gbl_lon, lon = ',gbl_lat(jj), lat(jy), gbl_lon(ii), lon(ix)
!      if (jy.eq.ny/2.and.ix.eq.nx/2) print*, 'hadj = ',hadj
      do nn = 1, nzgrid
        !*** TESTING 
 !       htest(ix,jy,nn) = halfh(nn)+hadj
        do kz = 1, nz
          if ( geoh(ix,jy,kz).gt.(halfh(nn)+hadj) ) exit
        end do
!        if(jy.eq.ny/2.and.ix.eq.nx/2) print*, 'kz, halfh(nn)+hadj, geoh(kz) = ',kz, halfh(nn)+hadj, geoh(ix,jy,kz)
        if ( kz.gt.nz ) then
          gridout(ix,jy,nn) = gridin(ix,jy,nz)  
        else if ( kz.gt.1 ) then
          dz1 = (halfh(nn) + hadj) - geoh(ix,jy,kz-1) 
          dz2 = geoh(ix,jy,kz) - (halfh(nn) + hadj)
          gridout(ix,jy,nn) = (gridin(ix,jy,kz-1)*dz2+gridin(ix,jy,kz)*dz1)/(dz1+dz2)
 !         if (jy.eq.ny/2.and.ix.eq.nx/2) print*, 'w1, w2 = ',dz1/(dz1+dz2), dz2/(dz1+dz2)
        else
          gridout(ix,jy,nn) = gridin(ix,jy,kz)
        endif 
      end do
    end do
  end do

  !*** TESTING
!  open(200,file='conc_profile_in.txt',action='write',status='replace')
!  do nn = 1, nz
!    write(200,fmt='(F8.2,1X,F8.2)') geoh(ix/2,jy/2,nn), gridin(ix/2,jy/2,nn)
!  end do
!  close(200)
!  open(200,file='conc_profile_out.txt',action='write',status='replace')
!  do nn = 1, nzgrid
!    write(200,fmt='(F8.2,1X,F8.2)') htest(ix/2,jy/2,nn), gridout(ix/2,jy/2,nn)
!  end do
!  close(200)

end subroutine interpz


